#include "Includes.h"

// CONFIG1H
#pragma config OSC = INTIO67    // Oscillator Selection bits (Internal oscillator block, port function on RA6 and RA7)
#pragma config FCMEN = ON      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRT = ON       // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOREN = ON  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
#pragma config BORV = 3         // Brown Out Reset Voltage bits (Minimum setting)

// CONFIG2H
#pragma config WDT = OFF         // Watchdog Timer Enable bit (WDT enabled)
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = PORTC   // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF      // PORTB A/D Enable bit (PORTB<4:0> pins are configured as analog input channels on Reset)
#pragma config LPT1OSC = OFF    // Low-Power Timer1 Oscillator Enable bit (Timer1 configured for higher power operation)
#pragma config MCLRE = OFF       // MCLR Pin Enable bit (MCLR pin enabled; RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF         // Single-Supply ICSP Enable bit (Single-Supply ICSP enabled)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 (000800-003FFFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 (004000-007FFFh) not code-protected)
#pragma config CP2 = OFF        // Code Protection bit (Block 2 (008000-00BFFFh) not code-protected)
#pragma config CP3 = OFF        // Code Protection bit (Block 3 (00C000-00FFFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (000800-003FFFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (004000-007FFFh) not write-protected)
#pragma config WRT2 = OFF       // Write Protection bit (Block 2 (008000-00BFFFh) not write-protected)
#pragma config WRT3 = OFF       // Write Protection bit (Block 3 (00C000-00FFFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0007FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (000800-003FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (004000-007FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection bit (Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection bit (Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0007FFh) not protected from table reads executed in other blocks)

__EEPROM_DATA(255,255,255,255,255,255,255,255);
__EEPROM_DATA(255,255,255,255,255,255,255,255);
__EEPROM_DATA(0,0,0,0,255,255,255,255);
__EEPROM_DATA(255,255,255,255,255,255,255,255);
__EEPROM_DATA(0,0,0,0,255,255,255,255);
__EEPROM_DATA(255,255,255,255,255,255,255,255);
__EEPROM_DATA(0,0,0,0,255,255,255,255);
__EEPROM_DATA(255,255,255,255,255,255,255,255);
__EEPROM_DATA(0,0,0,0,255,255,255,255);
__EEPROM_DATA(255,255,255,255,255,255,255,255);

#define OLED 0x78


// GLOBAL VARIABLES
unsigned int     zerofart=0, r1fart=0, digitr1fart=0, digitr2fart=0, r2fart=0, maxfart=0, digitmaxfart=0;
unsigned int     ADC_value0 = 0, digitADC=0, counter=180, digit1, digit2, digit3, digit4;
unsigned int     Memory1=0, Memory2=0, Memory3=0, Memory4=0;
unsigned char    ValStr[6];
unsigned char    txt[100];
int DISPLAY [1024];

void OLED_command(int ind, int com)
{
   startI2C ();
   sendI2C (ind);
   sendI2C (0x00);
   sendI2C (com);
   stopI2C ();
}

void OLED_write(int ind)
{
   long i;
   OLED_command (ind, 0x21) ;
   OLED_command (ind, 0x00);
   OLED_command (ind, 127);
   OLED_command (ind, 0x22) ;
   OLED_command (ind, 0x00);
   OLED_command (ind, 7);
   startI2C ();
   sendI2C (ind) ;
   sendI2C (0x40) ;
   for (i = 0; i < 1024; i++)
   {
     
      sendI2C (DISPLAY[i]) ;
   }

   stopI2C ();
}

void  OLED_init(int add)
{
   OLED_command (add, 0xae) ;
   OLED_command (add, 0xa8);
   OLED_command (add, 0x3f);
   OLED_command (add, 0xd3);
   OLED_command (add, 0x00);
   OLED_command (add, 0x40);
   
   OLED_command (add, 0xa0);
   OLED_command (add, 0xa1);
   OLED_command (add, 0xc0);
   
   OLED_command (add, 0xc8);
   OLED_command (add, 0xda);
   OLED_command (add, 0x12);
   OLED_command (add, 0x81);
   OLED_command (add, 0xfF);
   OLED_command (add, 0xa4);
   OLED_command (add, 0xa6) ;
   OLED_command (add, 0xd5);
   OLED_command (add, 0x80);
   OLED_command (add, 0x8d);
   OLED_command (add, 0x14) ;
   OLED_command (add, 0xAF) ;
   OLED_command (add, 0x20) ;
   OLED_command (add, 0x00) ;
}

void OLED_pixel(long x,long y)
{
   long nt;
   long page;
   long biti;
   page = y /8;
   biti= y-(page*8);
   nt= DISPLAY[page*128+x];
   nt |= 1 << biti;
   DISPLAY[page*128+x] = nt;
}

void OLED_text ( int x, int y, char* textptr, int size)
{
   unsigned char i, j, k, l, m;                  // Loop counters
   unsigned char pixelData [ 5 ];                  // Stores character data

   for ( i = 0; textptr [ i ] != 0x00; ++i, ++x )      // Loop through the passed string
   {
      if ( textptr [ i ] < 'S') // Checks if the letter is in the first text array
         memcpy ( pixelData, TEXT [ textptr [ i ] - ' '], 5 );
      
      else if(textptr[i] <= '~') // Check if the letter is in the second array
      {
         if ( ( textptr[i] >= 'a' ) && ( textptr[i] <= 'z' ) )
            memcpy ( pixelData, TEXT2 [ textptr [ i ] - 'S' - 1 ], 5 );
         
         else
            memcpy ( pixelData, TEXT2 [ textptr [ i ] - 'S' ], 5 );

         }
      
      else
         memcpy ( pixelData, TEXT [ 0 ], 5 );      // Default to space

      if ( x + 5 * size >= 128 )            // Performs character wrapping
      {   
         x = 0;                              // Set x at far left position
         y += 7*size + 1;                     // Set y at next position down
            }
      
      for ( j = 0; j < 5; ++j, x += size )         // Loop through character byte data
      {
         for ( k = 0; k <7 *size; ++k )            // Loop through the vertical pixels
         {
             if( (pixelData[j]&(1<< k))!= 0 ) // Check if the pixel should be set
            {
               for ( l = 0; l < size; ++l )      // The next two loops change the
               {                           // character's size
                  for ( m = 0; m < size; ++m )
                  {
                     OLED_pixel ( x + m, y + k * size + l); // Draws the pixel
                     }
                  }
               }
            }
         }
      }
   }

   void logo()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = LOGOS[x];
      }
   }
   void web()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = link[x];
      }
   }
   void goreny()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = skunk[x];
      }
   }
   void elso()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = winner1st[x];
      }
   }
   void masodik()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = winner2nd[x];
      }
   }
   void harmadik()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = winner3rd[x];
      }
   }
   void negyedik()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = winner4th[x];
      }
   }
   void OLED_clear()
   {
      long x;
      for (x = 0; x < 1024; x++)
      {
         DISPLAY[x] = 0x00;
      }
   }
 
   void delayMS(uint16 ms)
{
    uint16 k=0;
    
    for(k=0; k<ms; k++)
    {
        __delay_ms(1);
    }
}
   
   void detection()   
{
    counter =100;
   
     while(1)      
        {
            OLED_clear();   //Clear screen
            __delay_ms(50);
            counter =60;
            zerofart = GetADCValue0();
           
        while (2)
        {
            OLED_clear();
        r1fart = GetADCValue0();	// Read ADC value of CO
        digit1 = (unsigned int)(r1fart/1000);									 // Calculate digit1 of ADC_value
        digit2 = (unsigned int)((r1fart - digit1*1000)/100);					 // Calculate digit2 of ADC_value
        digit3 = (unsigned int)((r1fart - (digit1*1000+digit2*100))/10);		 // Calculate digit3 of ADC_value
        digit4 = (unsigned int)(r1fart - (digit1*1000+digit2*100+digit3*10));	 // Calculate digit4 of ADC_value
        digitr1fart = digit1*1000 + digit2*100 + digit3*10 + digit4;//Saved EEPROM CO digits, convert to decimal string to compare.
        ValStr[4]='\0';  
        if(digitr1fart>=1000)   { ValStr[3]=digitr1fart%10+48; digitr1fart = digitr1fart/10; } else ValStr[3]=32;
        if(digitr1fart>=100)    { ValStr[2]=digitr1fart%10+48; digitr1fart = digitr1fart/10; } else ValStr[2]=32;
        if(digitr1fart>=10)     { ValStr[1]=digitr1fart%10+48; digitr1fart = digitr1fart/10; } else ValStr[1]=32;
        ValStr[0] = digitr1fart+48; 
        OLED_text(15,0,(char*)"Detection",2);
        OLED_text(40,30,ValStr,3);
       
        counter--;          // Counter decrement
        delayMS(2000);
        r2fart = GetADCValue0();
        OLED_write(OLED);
       
        if (r1fart>=zerofart+100)
        {
            counter =45;
            if (r1fart > r2fart)
            {
                OLED_clear();
                maxfart = r1fart;
                
                PORTAbits.RA1=1; //buzzer
                delayMS(300);
                PORTAbits.RA1=0;
                delayMS(300);
                PORTAbits.RA1=1; //buzzer
                delayMS(300);
                PORTAbits.RA1=0;
                delayMS(300);
                PORTAbits.RA1=1; //buzzer
                delayMS(300);
                PORTAbits.RA1=0;
                
                if(maxfart>=300 && maxfart<500)
                {
                    OLED_text(10,32,(char*)"NOT BAD",2);
                    OLED_write(OLED);
                    delayMS(20000);
                }
                if(maxfart>=500 && maxfart<700)
                {
                    OLED_text(15,32,(char*)"OH GOD",2);
                    OLED_write(OLED);
                    delayMS(20000);
                }
                if(maxfart>=700 && maxfart<900)
                {
                    OLED_text(5,32,(char*)"HOLY SHIT",2);
                    OLED_write(OLED);
                    delayMS(20000);
                }
                if(maxfart>=900)
                {
                    OLED_text(0,32,(char*)"LIQUID FART",2);
                    OLED_write(OLED);
                    delayMS(20000);
                }
                
                
                while(3)           // do that while until counter is >0 , when counter will be 0  back to main while(1)
                {
                    OLED_clear();
                    digit1 = (unsigned int)(maxfart/1000);									 // Calculate digit1 of ADC_value
                    digit2 = (unsigned int)((maxfart - digit1*1000)/100);					 // Calculate digit2 of ADC_value
                    digit3 = (unsigned int)((maxfart - (digit1*1000+digit2*100))/10);		 // Calculate digit3 of ADC_value
                    digit4 = (unsigned int)(maxfart - (digit1*1000+digit2*100+digit3*10));	 // Calculate digit4 of ADC_value
                    digitmaxfart = digit1*1000 + digit2*100 + digit3*10 + digit4;//Saved EEPROM CO digits, convert to decimal string to compare.
                    ValStr[4]='\0';  
                    if(digitmaxfart>=1000)   { ValStr[3]=digitmaxfart%10+48; digitmaxfart = digitmaxfart/10; } else ValStr[3]=32;
                    if(digitmaxfart>=100)    { ValStr[2]=digitmaxfart%10+48; digitmaxfart = digitmaxfart/10; } else ValStr[2]=32;
                    if(digitmaxfart>=10)     { ValStr[1]=digitmaxfart%10+48; digitmaxfart = digitmaxfart/10; } else ValStr[1]=32;
                    ValStr[0] = digitmaxfart+48;
                    OLED_text(0,0,(char*)"YES      NO",2);
                    OLED_text(43,18,(char*)"SAVE",2);
                    OLED_text(0,52,(char*)"MAX:",1);
                    OLED_text(40,40,ValStr,3);
                    OLED_write(OLED);

                    __delay_ms(50);
                while(PORTBbits.RB1==0)                   // while UP button pressed
                {
                __delay_ms(50);             //0.075s delay
                if(PORTBbits.RB1==1)                    // If UP button released
                {
                    __delay_ms(50);
                    unsigned char State = m1;// start STATE with CO2 display
                    
                        while(4)
                        {
                            __delay_ms(50);
                            OLED_write(OLED);
                            while(PORTBbits.RB1==0)                   // while UP button pressed
                            {
                            __delay_ms(50);             //0.075s delay
                            if(PORTBbits.RB1==1)                    // If UP button released
                            {
                                
                                __delay_ms(50);        //0.1s delay
                                switch(State)           // Shift to next state
                                {
                                case m1:    State = m2;break;
                                case m2:	State = m3;break;
                                case m3:	State = m4;break;
                                case m4:	State = m1;break;
                                }
                            } 
                            }

                            if(State == m1)
                            {
                                OLED_clear();
                                Memory1 = EEPROM_Read(0x10)*1000 + EEPROM_Read(0x11)*100 + EEPROM_Read(0x12)*10 + EEPROM_Read(0x13);//Saved EEPROM CO digits, convert to decimal string to compare.
                                ValStr[4]='\0';  
                                if(Memory1>=1000)   { ValStr[3]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[3]=32;
                                if(Memory1>=100)    { ValStr[2]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[2]=32;
                                if(Memory1>=10)     { ValStr[1]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[1]=32;
                                ValStr[0] = Memory1+48;
                                OLED_text(0,3,(char*)"1st Memory :",1);
                                OLED_text(80,0,ValStr,2);

                                while(PORTBbits.RB0==0)                   // while UP button pressed
                                {
                                    EEPROM_Write(0x10,digit1);         //save CO 1st digit to 1st EEPROM address
                                    EEPROM_Write(0x11,digit2);         //save CO 2nd digit to 2nd EEPROM address
                                    EEPROM_Write(0x12,digit3);         //save CO 3rd digit to 3rd EEPROM address
                                    EEPROM_Write(0x13,digit4);         //save CO 4th digit to 4th EEPROM address
                                    if(PORTBbits.RB0==1)                    // If UP button released
                                    {
                                        OLED_clear();
                                        Memory1 = EEPROM_Read(0x10)*1000 + EEPROM_Read(0x11)*100 + EEPROM_Read(0x12)*10 + EEPROM_Read(0x13);//Saved EEPROM CO digits, convert to decimal string to compare.
                                        ValStr[4]='\0';  
                                        if(Memory1>=1000)   { ValStr[3]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[3]=32;
                                        if(Memory1>=100)    { ValStr[2]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[2]=32;
                                        if(Memory1>=10)     { ValStr[1]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[1]=32;
                                        ValStr[0] = Memory1+48;
                                        OLED_text(0,3,(char*)"SAVED      :",1);
                                        OLED_text(80,0,ValStr,2);
                                        OLED_write(OLED);
                                        delayMS(10000);
                                        return;
                                    }
                                }
                            }
                            if(State == m2)
                            {
                                OLED_clear();
                                Memory2 = EEPROM_Read(0x20)*1000 + EEPROM_Read(0x21)*100 + EEPROM_Read(0x22)*10 + EEPROM_Read(0x23);//Saved EEPROM CO digits, convert to decimal string to compare.
                                ValStr[4]='\0';  
                                if(Memory2>=1000)   { ValStr[3]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[3]=32;
                                if(Memory2>=100)    { ValStr[2]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[2]=32;
                                if(Memory2>=10)     { ValStr[1]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[1]=32;
                                ValStr[0] = Memory2+48;
                                OLED_text(0,19,(char*)"2nd Memory :",1);
                                OLED_text(80,16,ValStr,2);

                                while(PORTBbits.RB0==0)                   // while UP button pressed
                                {
                                    EEPROM_Write(0x20,digit1);         //save CO 1st digit to 1st EEPROM address
                                    EEPROM_Write(0x21,digit2);         //save CO 2nd digit to 2nd EEPROM address
                                    EEPROM_Write(0x22,digit3);         //save CO 3rd digit to 3rd EEPROM address
                                    EEPROM_Write(0x23,digit4);         //save CO 4th digit to 4th EEPROM address
                                    if(PORTBbits.RB0==1)                    // If UP button released
                                    {
                                        OLED_clear();
                                        Memory2 = EEPROM_Read(0x20)*1000 + EEPROM_Read(0x21)*100 + EEPROM_Read(0x22)*10 + EEPROM_Read(0x23);//Saved EEPROM CO digits, convert to decimal string to compare.
                                        ValStr[4]='\0';  
                                        if(Memory2>=1000)   { ValStr[3]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[3]=32;
                                        if(Memory2>=100)    { ValStr[2]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[2]=32;
                                        if(Memory2>=10)     { ValStr[1]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[1]=32;
                                        ValStr[0] = Memory2+48;
                                        OLED_text(0,19,(char*)"SAVED      :",1);
                                        OLED_text(80,16,ValStr,2);
                                        OLED_write(OLED);
                                        delayMS(10000);
                                        return;
                                    }
                                }
                            }
                            if(State == m3)
                            {
                                OLED_clear();
                                Memory3 = EEPROM_Read(0x30)*1000 + EEPROM_Read(0x31)*100 + EEPROM_Read(0x32)*10 + EEPROM_Read(0x33);//Saved EEPROM CO digits, convert to decimal string to compare.
                                ValStr[4]='\0';  
                                if(Memory3>=1000)   { ValStr[3]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[3]=32;
                                if(Memory3>=100)    { ValStr[2]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[2]=32;
                                if(Memory3>=10)     { ValStr[1]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[1]=32;
                                ValStr[0] = Memory3+48;
                                OLED_text(0,35,(char*)"3rd Memory :",1);
                                OLED_text(80,32,ValStr,2);

                                while(PORTBbits.RB0==0)                   // while UP button pressed
                                {
                                    EEPROM_Write(0x30,digit1);         //save CO 1st digit to 1st EEPROM address
                                    EEPROM_Write(0x31,digit2);         //save CO 2nd digit to 2nd EEPROM address
                                    EEPROM_Write(0x32,digit3);         //save CO 3rd digit to 3rd EEPROM address
                                    EEPROM_Write(0x33,digit4);         //save CO 4th digit to 4th EEPROM address
                                    if(PORTBbits.RB0==1)                    // If UP button released
                                    {
                                        OLED_clear();
                                        Memory3 = EEPROM_Read(0x30)*1000 + EEPROM_Read(0x31)*100 + EEPROM_Read(0x32)*10 + EEPROM_Read(0x33);//Saved EEPROM CO digits, convert to decimal string to compare.
                                        ValStr[4]='\0';  
                                        if(Memory3>=1000)   { ValStr[3]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[3]=32;
                                        if(Memory3>=100)    { ValStr[2]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[2]=32;
                                        if(Memory3>=10)     { ValStr[1]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[1]=32;
                                        ValStr[0] = Memory3+48;
                                        OLED_text(0,35,(char*)"SAVED      :",1);
                                        OLED_text(80,32,ValStr,2);
                                        OLED_write(OLED);
                                        delayMS(10000);
                                        return;
                                    }
                                }
                            }
                            if(State == m4)
                            {
                                OLED_clear();
                                Memory4 = EEPROM_Read(0x40)*1000 + EEPROM_Read(0x41)*100 + EEPROM_Read(0x42)*10 + EEPROM_Read(0x43);//Saved EEPROM CO digits, convert to decimal string to compare.
                                ValStr[4]='\0';  
                                if(Memory4>=1000)   { ValStr[3]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[3]=32;
                                if(Memory4>=100)    { ValStr[2]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[2]=32;
                                if(Memory4>=10)     { ValStr[1]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[1]=32;
                                ValStr[0] = Memory4+48;
                                OLED_text(0,51,(char*)"4th Memory :",1);
                                OLED_text(80,48,ValStr,2);

                                while(PORTBbits.RB0==0)                   // while UP button pressed
                                {
                                    EEPROM_Write(0x40,digit1);         //save CO 1st digit to 1st EEPROM address
                                    EEPROM_Write(0x41,digit2);         //save CO 2nd digit to 2nd EEPROM address
                                    EEPROM_Write(0x42,digit3);         //save CO 3rd digit to 3rd EEPROM address
                                    EEPROM_Write(0x43,digit4);         //save CO 4th digit to 4th EEPROM address
                                    if(PORTBbits.RB0==1)                    // If UP button released
                                    {
                                        OLED_clear();
                                        Memory4 = EEPROM_Read(0x40)*1000 + EEPROM_Read(0x41)*100 + EEPROM_Read(0x42)*10 + EEPROM_Read(0x43);//Saved EEPROM CO digits, convert to decimal string to compare.
                                        ValStr[4]='\0';  
                                        if(Memory4>=1000)   { ValStr[3]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[3]=32;
                                        if(Memory4>=100)    { ValStr[2]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[2]=32;
                                        if(Memory4>=10)     { ValStr[1]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[1]=32;
                                        ValStr[0] = Memory4+48;
                                        OLED_text(0,51,(char*)"SAVED      :",1);
                                        OLED_text(80,48,ValStr,2);
                                        OLED_write(OLED);
                                        delayMS(10000);
                                        return;
                                    }
                                }
                            }
                        }
                }
                }

                while(PORTBbits.RB0==0)                   // while UP button pressed
                {
                    __delay_ms(50);             //0.075s delay
                    if(PORTBbits.RB0==1)                    // If UP button released
                    {
                        __delay_ms(50);             //0.075s delay
                        return;
                    } 
                }
                }
                }
            }
         
        
        if(counter==0)      //if counter = 0
        {
            OLED_clear();   //Clear screen
            OLED_text(15,0,(char*)"Detection",2);
            OLED_text(15,35,(char*)"Time out",2);
            OLED_write(OLED);
            PORTAbits.RA1=1; //buzzer
            delayMS(750);
            PORTAbits.RA1=0;
            delayMS(6000);
            
            return;     //back to main while(1)
        }
        } 
        }
}
   
   void delete_menu()
   {
    __delay_ms(50);
    counter =100;  
    OLED_clear();
    
        while(1)           // do that while until counter is >0 , when counter will be 0  back to main while(1)
        {
            __delay_ms(50);     // delay 0.03s
            
            
            OLED_text(0,0,(char*)"YES      NO",2);
            OLED_text(28,25,(char*)"DELETE",2);
            OLED_text(0,45,(char*)"Your Score?",2);
            OLED_write(OLED);
            
            while(PORTBbits.RB1==0)                   // while UP button pressed
            {
            __delay_ms(50);             //0.075s delay
            if(PORTBbits.RB1==1)                    // If UP button released
            {
                __delay_ms(50);
                unsigned char State = m1;// start STATE with CO2 display

                    while(2)
                    {
                        __delay_ms(50);
                        OLED_write(OLED);
                        counter--;          // Counter decrement
                        while(PORTBbits.RB1==0)                   // while UP button pressed
                        {
                        __delay_ms(50);             //0.075s delay
                        if(PORTBbits.RB1==1)                    // If UP button released
                        {
                            counter =75;
                            __delay_ms(50);        //0.1s delay
                            switch(State)           // Shift to next state
                            {
                            case m1:    State = m2;break;
                            case m2:	State = m3;break;
                            case m3:	State = m4;break;
                            case m4:	State = m1;break;
                            }
                        } 
                        }

                        if(State == m1)
                        {
                            OLED_clear();
                            Memory1 = EEPROM_Read(0x10)*1000 + EEPROM_Read(0x11)*100 + EEPROM_Read(0x12)*10 + EEPROM_Read(0x13);//Saved EEPROM CO digits, convert to decimal string to compare.
                            ValStr[4]='\0';  
                            if(Memory1>=1000)   { ValStr[3]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[3]=32;
                            if(Memory1>=100)    { ValStr[2]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[2]=32;
                            if(Memory1>=10)     { ValStr[1]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[1]=32;
                            ValStr[0] = Memory1+48;
                            OLED_text(0,3,(char*)"1st Memory :",1);
                            OLED_text(80,0,ValStr,2);

                            while(PORTBbits.RB0==0)                   // while UP button pressed
                            {
                                EEPROM_Write(0x10,0);         //save CO 1st digit to 1st EEPROM address
                                EEPROM_Write(0x11,0);         //save CO 2nd digit to 2nd EEPROM address
                                EEPROM_Write(0x12,0);         //save CO 3rd digit to 3rd EEPROM address
                                EEPROM_Write(0x13,0);         //save CO 4th digit to 4th EEPROM address
                                if(PORTBbits.RB0==1)                    // If UP button released
                                {
                                    OLED_clear();
                                    Memory1 = EEPROM_Read(0x10)*1000 + EEPROM_Read(0x11)*100 + EEPROM_Read(0x12)*10 + EEPROM_Read(0x13);//Saved EEPROM CO digits, convert to decimal string to compare.
                                    ValStr[4]='\0';  
                                    if(Memory1>=1000)   { ValStr[3]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[3]=32;
                                    if(Memory1>=100)    { ValStr[2]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[2]=32;
                                    if(Memory1>=10)     { ValStr[1]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[1]=32;
                                    ValStr[0] = Memory1+48;
                                    OLED_text(0,3,(char*)"DELETED    :",1);
                                    OLED_text(80,0,ValStr,2);
                                    OLED_write(OLED);
                                    delayMS(10000);
                                    return;
                                }
                            }
                        }
                        if(State == m2)
                        {
                            OLED_clear();
                            Memory2 = EEPROM_Read(0x20)*1000 + EEPROM_Read(0x21)*100 + EEPROM_Read(0x22)*10 + EEPROM_Read(0x23);//Saved EEPROM CO digits, convert to decimal string to compare.
                            ValStr[4]='\0';  
                            if(Memory2>=1000)   { ValStr[3]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[3]=32;
                            if(Memory2>=100)    { ValStr[2]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[2]=32;
                            if(Memory2>=10)     { ValStr[1]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[1]=32;
                            ValStr[0] = Memory2+48;
                            OLED_text(0,19,(char*)"2nd Memory :",1);
                            OLED_text(80,16,ValStr,2);

                            while(PORTBbits.RB0==0)                   // while UP button pressed
                            {
                                EEPROM_Write(0x20,0);         //save CO 1st digit to 1st EEPROM address
                                EEPROM_Write(0x21,0);         //save CO 2nd digit to 2nd EEPROM address
                                EEPROM_Write(0x22,0);         //save CO 3rd digit to 3rd EEPROM address
                                EEPROM_Write(0x23,0);         //save CO 4th digit to 4th EEPROM address
                                if(PORTBbits.RB0==1)                    // If UP button released
                                {
                                    OLED_clear();
                                    Memory2 = EEPROM_Read(0x20)*1000 + EEPROM_Read(0x21)*100 + EEPROM_Read(0x22)*10 + EEPROM_Read(0x23);//Saved EEPROM CO digits, convert to decimal string to compare.
                                    ValStr[4]='\0';  
                                    if(Memory2>=1000)   { ValStr[3]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[3]=32;
                                    if(Memory2>=100)    { ValStr[2]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[2]=32;
                                    if(Memory2>=10)     { ValStr[1]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[1]=32;
                                    ValStr[0] = Memory2+48;
                                    OLED_text(0,19,(char*)"DELETED    :",1);
                                    OLED_text(80,16,ValStr,2);
                                    OLED_write(OLED);
                                    delayMS(10000);
                                    return;
                                }
                            }
                        }
                        if(State == m3)
                        {
                            OLED_clear();
                            Memory3 = EEPROM_Read(0x30)*1000 + EEPROM_Read(0x31)*100 + EEPROM_Read(0x32)*10 + EEPROM_Read(0x33);//Saved EEPROM CO digits, convert to decimal string to compare.
                            ValStr[4]='\0';  
                            if(Memory3>=1000)   { ValStr[3]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[3]=32;
                            if(Memory3>=100)    { ValStr[2]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[2]=32;
                            if(Memory3>=10)     { ValStr[1]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[1]=32;
                            ValStr[0] = Memory3+48;
                            OLED_text(0,35,(char*)"3rd Memory :",1);
                            OLED_text(80,32,ValStr,2);

                            while(PORTBbits.RB0==0)                   // while UP button pressed
                            {
                                EEPROM_Write(0x30,0);         //save CO 1st digit to 1st EEPROM address
                                EEPROM_Write(0x31,0);         //save CO 2nd digit to 2nd EEPROM address
                                EEPROM_Write(0x32,0);         //save CO 3rd digit to 3rd EEPROM address
                                EEPROM_Write(0x33,0);         //save CO 4th digit to 4th EEPROM address
                                if(PORTBbits.RB0==1)                    // If UP button released
                                {
                                    OLED_clear();
                                    Memory3 = EEPROM_Read(0x30)*1000 + EEPROM_Read(0x31)*100 + EEPROM_Read(0x32)*10 + EEPROM_Read(0x33);//Saved EEPROM CO digits, convert to decimal string to compare.
                                    ValStr[4]='\0';  
                                    if(Memory3>=1000)   { ValStr[3]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[3]=32;
                                    if(Memory3>=100)    { ValStr[2]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[2]=32;
                                    if(Memory3>=10)     { ValStr[1]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[1]=32;
                                    ValStr[0] = Memory3+48;
                                    OLED_text(0,35,(char*)"DELETED    :",1);
                                    OLED_text(80,32,ValStr,2);
                                    OLED_write(OLED);
                                    delayMS(10000);
                                    return;
                                }
                            }
                        }
                        if(State == m4)
                        {
                            OLED_clear();
                            Memory4 = EEPROM_Read(0x40)*1000 + EEPROM_Read(0x41)*100 + EEPROM_Read(0x42)*10 + EEPROM_Read(0x43);//Saved EEPROM CO digits, convert to decimal string to compare.
                            ValStr[4]='\0';  
                            if(Memory4>=1000)   { ValStr[3]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[3]=32;
                            if(Memory4>=100)    { ValStr[2]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[2]=32;
                            if(Memory4>=10)     { ValStr[1]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[1]=32;
                            ValStr[0] = Memory4+48;
                            OLED_text(0,51,(char*)"4th Memory :",1);
                            OLED_text(80,48,ValStr,2);

                            while(PORTBbits.RB0==0)                   // while UP button pressed
                            {
                                EEPROM_Write(0x40,0);         //save CO 1st digit to 1st EEPROM address
                                EEPROM_Write(0x41,0);         //save CO 2nd digit to 2nd EEPROM address
                                EEPROM_Write(0x42,0);         //save CO 3rd digit to 3rd EEPROM address
                                EEPROM_Write(0x43,0);         //save CO 4th digit to 4th EEPROM address
                                if(PORTBbits.RB0==1)                    // If UP button released
                                {
                                    OLED_clear();
                                    Memory4 = EEPROM_Read(0x40)*1000 + EEPROM_Read(0x41)*100 + EEPROM_Read(0x42)*10 + EEPROM_Read(0x43);//Saved EEPROM CO digits, convert to decimal string to compare.
                                    ValStr[4]='\0';  
                                    if(Memory4>=1000)   { ValStr[3]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[3]=32;
                                    if(Memory4>=100)    { ValStr[2]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[2]=32;
                                    if(Memory4>=10)     { ValStr[1]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[1]=32;
                                    ValStr[0] = Memory4+48;
                                    OLED_text(0,51,(char*)"DELETED    :",1);
                                    OLED_text(80,48,ValStr,2);
                                    OLED_write(OLED);
                                    delayMS(10000);
                                    return;
                                }
                            }
                        }
                        
                        if(counter==0)                          //if counter = 0
                        {
                            return;                              //back to main while(1)
                        }
                    }
            }
            }
            
            while(PORTBbits.RB0==0)                   // while UP button pressed
            {
                __delay_ms(50);             //0.075s delay
                if(PORTBbits.RB0==1)                    // If UP button released
                {
                    __delay_ms(50);             //0.075s delay
                    return;
                } 
            }
        }   
   }
   
   void scores()
   {
       while(1)
       {
           
       
       OLED_clear();
           
        Memory1 = EEPROM_Read(0x10)*1000 + EEPROM_Read(0x11)*100 + EEPROM_Read(0x12)*10 + EEPROM_Read(0x13);//Saved EEPROM CO digits, convert to decimal string to compare.
        ValStr[4]='\0';  
        if(Memory1>=1000)   { ValStr[3]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[3]=32;
        if(Memory1>=100)    { ValStr[2]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[2]=32;
        if(Memory1>=10)     { ValStr[1]=Memory1%10+48; Memory1 = Memory1/10; } else ValStr[1]=32;
        ValStr[0] = Memory1+48;
        OLED_text(0,3,(char*)"1st Memory :",1);
        OLED_text(80,0,ValStr,2);

        Memory2 = EEPROM_Read(0x20)*1000 + EEPROM_Read(0x21)*100 + EEPROM_Read(0x22)*10 + EEPROM_Read(0x23);//Saved EEPROM CO digits, convert to decimal string to compare.
        ValStr[4]='\0';  
        if(Memory2>=1000)   { ValStr[3]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[3]=32;
        if(Memory2>=100)    { ValStr[2]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[2]=32;
        if(Memory2>=10)     { ValStr[1]=Memory2%10+48; Memory2 = Memory2/10; } else ValStr[1]=32;
        ValStr[0] = Memory2+48;
        OLED_text(0,19,(char*)"2nd Memory :",1);
        OLED_text(80,16,ValStr,2);

        Memory3 = EEPROM_Read(0x30)*1000 + EEPROM_Read(0x31)*100 + EEPROM_Read(0x32)*10 + EEPROM_Read(0x33);//Saved EEPROM CO digits, convert to decimal string to compare.
        ValStr[4]='\0';  
        if(Memory3>=1000)   { ValStr[3]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[3]=32;
        if(Memory3>=100)    { ValStr[2]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[2]=32;
        if(Memory3>=10)     { ValStr[1]=Memory3%10+48; Memory3 = Memory3/10; } else ValStr[1]=32;
        ValStr[0] = Memory3+48;
        OLED_text(0,35,(char*)"3rd Memory :",1);
        OLED_text(80,32,ValStr,2);

        Memory4 = EEPROM_Read(0x40)*1000 + EEPROM_Read(0x41)*100 + EEPROM_Read(0x42)*10 + EEPROM_Read(0x43);//Saved EEPROM CO digits, convert to decimal string to compare.
        ValStr[4]='\0';  
        if(Memory4>=1000)   { ValStr[3]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[3]=32;
        if(Memory4>=100)    { ValStr[2]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[2]=32;
        if(Memory4>=10)     { ValStr[1]=Memory4%10+48; Memory4 = Memory4/10; } else ValStr[1]=32;
        ValStr[0] = Memory4+48;
        OLED_text(0,51,(char*)"4th Memory :",1);
        OLED_text(80,48,ValStr,2);

        OLED_write(OLED);
        
        while(PORTBbits.RB1==0)                   // while UP button pressed
        {
            __delay_ms(50);             //0.075s delay
            if(PORTBbits.RB1==1)                    // If UP button released
            {
                return;
            } 
        }
        
        while(PORTBbits.RB0==0)                   // while UP button pressed
        {
            if(Memory1>Memory2 && Memory1>Memory3 && Memory1>Memory4)
            {
                OLED_clear();
                elso();
                OLED_write(OLED);
            }
            if(Memory2>Memory1 && Memory2>Memory3 && Memory2>Memory4)
            {
                OLED_clear();
                masodik();
                OLED_write(OLED);
            }
            if(Memory3>Memory1 && Memory3>Memory2 && Memory3>Memory4)
            {
                OLED_clear();
                harmadik();
                OLED_write(OLED);
            }
            if(Memory4>Memory1 && Memory4>Memory2 && Memory4>Memory3)
            {
                OLED_clear();
                negyedik();
                OLED_write(OLED);
            }
                
        }
        
       }
   }
  
   void welcome()
   {
    __delay_ms(50);
    counter =181;          
        while(counter > 0)           // do that while until counter is >0 , when counter will be 0  back to main while(1)
        {
            __delay_ms(50);     // delay 0.03s
            counter--;          // Counter decrement
            
            if(counter>180)                          //if counter = 0
            {
//                web();//logo();
//                OLED_write(OLED);
            }
            if(counter<180 && counter>150)                          //if counter = 0
            {
                OLED_clear();
                OLED_text(0,0,(char*)"FART sensor",2);
                OLED_text(0,16,(char*)"designed by",2);
                OLED_text(25,32,(char*)"Laszlo",2);
                OLED_text(25,48,(char*)"Kovacs",2);
                OLED_write(OLED);
            }
            if(counter<150 && counter>130)                          //if counter = 0
            {
                OLED_clear();
                OLED_text(0,0,(char*)"Please wait",2);
                OLED_text(15,25,(char*)"a minute",2);
                OLED_text(35,50,(char*)"until",2);
                OLED_write(OLED);
            }
            if(counter<130 && counter>110)                          //if counter = 0
            {
                OLED_clear();
                OLED_text(0,0,(char*)"The sensor",2);
                OLED_text(50,25,(char*)"is",2);
                OLED_text(0,50,(char*)"warming up",2);
                OLED_write(OLED);
            }
            if(counter<110 && counter>60)                          //if counter = 0
            {
                goreny();
                OLED_write(OLED);
            }
            if(counter<60 && counter>1)                          //if counter = 0
            {
                web();
                OLED_write(OLED);
            }

            if(counter==0)                          //if counter = 0
            {
                return;                              //back to main while(1)
            }
        }   
   }


void main()
{
    initI2C();
    __delay_ms(50);
    
    OLED_init(OLED);
    InitADC0();
    
    OSCCON=0b01110000;    //Set FOSC to 8MHZ
    OSCTUNEbits.PLLEN=1;  //x4 MHz
    TRISA=0b00000001;       // PORTA input RA0
    PORTA=0b00000000;       // PORTA low
    TRISB=0b00000011;       // PORTB input RB0,RB1
    PORTB=0b00000000;       // PORTB low
    //TRISC=0b00000000;       // PORTB input RB0,RB1
    //PORTC=0b00000000;       // PORTB low
    INTCON2bits.RBPU=0;     //pull ups
    
    welcome();
    
      while (1)
      {
        OLED_clear();
        
        ADC_value0 = GetADCValue0();
        
        digit1 = (unsigned int)(ADC_value0/1000);									 // Calculate digit1 of ADC_value
		digit2 = (unsigned int)((ADC_value0 - digit1*1000)/100);					 // Calculate digit2 of ADC_value
		digit3 = (unsigned int)((ADC_value0 - (digit1*1000+digit2*100))/10);		 // Calculate digit3 of ADC_value
		digit4 = (unsigned int)(ADC_value0 - (digit1*1000+digit2*100+digit3*10));	 // Calculate digit4 of ADC_value
        digitADC = digit1*1000 + digit2*100 + digit3*10 + digit4;//Saved EEPROM CO digits, convert to decimal string to compare.
        ValStr[4]='\0';  
        if(digitADC>=1000)   { ValStr[3]=digitADC%10+48; digitADC = digitADC/10; } else ValStr[3]=32;
        if(digitADC>=100)    { ValStr[2]=digitADC%10+48; digitADC = digitADC/10; } else ValStr[2]=32;
        if(digitADC>=10)     { ValStr[1]=digitADC%10+48; digitADC = digitADC/10; } else ValStr[1]=32;
        ValStr[0] = digitADC+48;
        OLED_text(10,0,(char*)"Fart score",2);
        OLED_text(40,30,ValStr,3);
        
        OLED_write(OLED);
        
        while(PORTBbits.RB0==0)                   // while UP button pressed
        {
            __delay_ms(50);             //0.075s delay
            if(PORTBbits.RB0==1)                    // If UP button released
            {
                detection();
            } 
        }
       
        
        while(PORTBbits.RB1==0)
        {
            __delay_ms(50);             //0.075s delay
            if(PORTBbits.RB1==1)                    // If UP button released
            {
                scores();
            } 
            
            if(PORTBbits.RB0==0)
            {
                OLED_clear();
                OLED_text(0,0,(char*)"YES      NO",2);
                OLED_text(28,25,(char*)"DELETE",2);
                OLED_text(0,45,(char*)"Your Score?",2);
                OLED_write(OLED);
                delayMS(1000);
                delete_menu();
            }
        }
        
      }
} 

